#!/usr/bin/env python

import sys
import getopt
from xml.dom import minidom
from xml.dom import Node
#import yaml

#
# If you have installed IPython you can uncomment and use the following.
# IPython is available from http://www-hep.colorado.edu/~fperez/ipython.
#
#from IPython.Shell import IPythonShellEmbed
#IPShell = IPythonShellEmbed('-nosep',
#    banner = 'Entering interpreter.  Ctrl-D to exit.',
#    exit_msg = 'Leaving Interpreter.')

# Use the following line where and when you want to drop into the
# IPython shell:
#    IPShell(vars(), '<a msg>')

#
# Support/utility functions.
#

def showIndent(outfile, level):
    for idx in range(level):
        outfile.write('    ')


#
# Data representation classes.
#

class workflow:
    subclass = None
    def __init__(self, name='', state_initial='', states=None):
        self.name = name
        self.state_initial = state_initial
        if states is None:
            self.states = []
        else:
            self.states = states
    def factory(*args):
        if workflow.subclass:
            return apply(workflow.subclass, args)
        else:
            return apply(workflow, args)
    factory = staticmethod(factory)
    def getName(self): return self.name
    def setName(self, name): self.name = name
    def getState_initial(self): return self.state_initial
    def setState_initial(self, state_initial): self.state_initial = state_initial
    def getStates(self): return self.states
    def addStates(self, states): self.states.append(states)
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<workflow>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<name>%s</name>\n' % self.getName())
        showIndent(outfile, level)
        outfile.write('<state_initial>%s</state_initial>\n' % self.getState_initial())
        for states in self.states:
            states.export(outfile, level)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</workflow>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'name':
                name = ''
                for text_ in child.childNodes:
                    name += text_.nodeValue
                self.name = name
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'state_initial':
                state_initial = ''
                for text_ in child.childNodes:
                    state_initial += text_.nodeValue
                self.state_initial = state_initial
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'state':
                obj = state.factory()
                obj.build(child)
                self.states.append(obj)
# end class workflow


class state:
    subclass = None
    def __init__(self, id='', transitions=None):
        self.id = id
        if transitions is None:
            self.transitions = []
        else:
            self.transitions = transitions
    def factory(*args):
        if state.subclass:
            return apply(state.subclass, args)
        else:
            return apply(state, args)
    factory = staticmethod(factory)
    def getId(self): return self.id
    def setId(self, id): self.id = id
    def getTransitions(self): return self.transitions
    def addTransitions(self, transitions): self.transitions.append(transitions)
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<state>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<id>%s</id>\n' % self.getId())
        for transitions in self.transitions:
            transitions.export(outfile, level)
        level -= 1
        showIndent(outfile, level)
        outfile.write('</state>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'id':
                id = ''
                for text_ in child.childNodes:
                    id += text_.nodeValue
                self.id = id
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'transition':
                obj = transition.factory()
                obj.build(child)
                self.transitions.append(obj)
# end class state


class transition:
    subclass = None
    def __init__(self, id='', state_from='', state_to='', condition='', action=''):
        self.id = id
        self.state_from = state_from
        self.state_to = state_to
        self.condition = condition
        self.action = action
    def factory(*args):
        if transition.subclass:
            return apply(transition.subclass, args)
        else:
            return apply(transition, args)
    factory = staticmethod(factory)
    def getId(self): return self.id
    def setId(self, id): self.id = id
    def getState_from(self): return self.state_from
    def setState_from(self, state_from): self.state_from = state_from
    def getState_to(self): return self.state_to
    def setState_to(self, state_to): self.state_to = state_to
    def getCondition(self): return self.condition
    def setCondition(self, condition): self.condition = condition
    def getAction(self): return self.action
    def setAction(self, action): self.action = action
    def export(self, outfile, level):
        showIndent(outfile, level)
        outfile.write('<transition>\n')
        level += 1
        showIndent(outfile, level)
        outfile.write('<id>%s</id>\n' % self.getId())
        showIndent(outfile, level)
        outfile.write('<state_from>%s</state_from>\n' % self.getState_from())
        showIndent(outfile, level)
        outfile.write('<state_to>%s</state_to>\n' % self.getState_to())
        showIndent(outfile, level)
        outfile.write('<condition>%s</condition>\n' % self.getCondition())
        showIndent(outfile, level)
        outfile.write('<action>%s</action>\n' % self.getAction())
        level -= 1
        showIndent(outfile, level)
        outfile.write('</transition>\n')
    def build(self, node_):
        attrs = node_.attributes
        for child in node_.childNodes:
            if child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'id':
                id = ''
                for text_ in child.childNodes:
                    id += text_.nodeValue
                self.id = id
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'state_from':
                state_from = ''
                for text_ in child.childNodes:
                    state_from += text_.nodeValue
                self.state_from = state_from
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'state_to':
                state_to = ''
                for text_ in child.childNodes:
                    state_to += text_.nodeValue
                self.state_to = state_to
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'condition':
                condition = ''
                for text_ in child.childNodes:
                    condition += text_.nodeValue
                self.condition = condition
            elif child.nodeType == Node.ELEMENT_NODE and \
                child.nodeName == 'action':
                action = ''
                for text_ in child.childNodes:
                    action += text_.nodeValue
                self.action = action
# end class transition


USAGE_TEXT = """
Usage: python <Parser>.py <in_xml_file>
"""

def usage():
    print USAGE_TEXT
    sys.exit(-1)


def parse(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.childNodes[0]
    rootObj = workflow.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('?xml version="1.0" ?>')
    rootObj.export(sys.stdout, 0)
    return rootObj


def parseString(inString):
    doc = minidom.parseString(inString)
    rootNode = doc.childNodes[0]
    rootObj = workflow.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout, 0)
    return rootObj


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        usage()
    parse(args[0])


if __name__ == '__main__':
    main()
    #import pdb
    #pdb.run('main()')

